@@ -14,6 +14,7 @@
 (require 'cl-lib)  ; For `cl-subseq`
 (require 'imenu)
 (require 'magit)
+(require 'subr-x)
 
 (declare-function helm-comp-read "helm-mode" (prompt collection &rest args))
 (declare-function ai-code-backends-infra--session-buffer-p "ai-code-backends-infra" (buffer))
@@ -37,8 +38,14 @@ This function combines candidate-list with history for better completion."
                      nil nil initial-input
                      'ai-code-read-string-history)))
 
+(defvar ai-code--read-string-fn #'ai-code-plain-read-string
+  "Function used by `ai-code-read-string' to read user input.")
+
 ;;;###autoload
-(defalias 'ai-code-read-string #'ai-code-plain-read-string)
+(defun ai-code-read-string (prompt &optional initial-input candidate-list)
+  "Read a string from the user with PROMPT and optional INITIAL-INPUT.
+CANDIDATE-LIST provides additional completion options if provided."
+  (funcall ai-code--read-string-fn prompt initial-input candidate-list))
 
 (defun ai-code-helm-read-string-with-history (prompt history-file-name &optional initial-input candidate-list)
   "Read a string with Helm completion using specified history file.
@@ -100,8 +107,11 @@ CANDIDATE-LIST is an optional list of candidate strings to show before history."
   (ai-code-helm-read-string-with-history prompt "ai-code-helm-read-string-history.el" initial-input candidate-list))
 
 ;;;###autoload
-(if (featurep 'helm)
-    (defalias 'ai-code-read-string #'ai-code-helm-read-string))
+(when (featurep 'helm)
+  (setq ai-code--read-string-fn #'ai-code-helm-read-string))
+
+(with-eval-after-load 'helm
+  (setq ai-code--read-string-fn #'ai-code-helm-read-string))
 
 (defun ai-code--get-window-files ()
   "Get a list of unique file paths from all visible windows."
@@ -125,19 +135,71 @@ The current buffer's file is always first."
           (concat "\nFiles:\n" (mapconcat #'identity sorted-files "\n"))
         ""))))
 
+(defun ai-code--imenu-subalist-p (payload)
+  "Return non-nil when PAYLOAD looks like an imenu sub-alist."
+  (and (listp payload)
+       (cl-some (lambda (entry)
+                  (and (consp entry) (stringp (car entry))))
+                payload)))
+
+(defun ai-code--imenu-item-position (payload)
+  "Extract buffer position from imenu PAYLOAD."
+  (cond
+   ((or (integerp payload) (markerp payload)) payload)
+   ((overlayp payload) (overlay-start payload))
+   ((and (consp payload)
+         (or (integerp (car payload))
+             (markerp (car payload))))
+    (car payload))
+   (t nil)))
+
+(defun ai-code--extract-symbol-from-line (line)
+  "Extract a likely symbol identifier from LINE."
+  (let ((patterns
+         '("^[ \t]*\\(?:async[ \t]+\\)?\\(?:def\\|class\\|function\\|func\\|fn\\|sub\\|proc\\|method\\|interface\\|struct\\|enum\\|type\\|trait\\|module\\|namespace\\)[ \t]+\\([[:word:]_.$:\\-]+\\)"
+           "^[ \t]*\\([[:word:]_.$:\\-]+\\)[ \t]*("
+           "^[ \t]*\\([[:word:]_.$:\\-]+\\)[ \t]*[{:]")))
+    (catch 'found
+      (dolist (pattern patterns)
+        (when (string-match pattern line)
+          (throw 'found (match-string 1 line))))
+      nil)))
+
+(defun ai-code--imenu-symbol-from-position (payload)
+  "Extract a symbol name from PAYLOAD position as fallback."
+  (when-let ((pos (ai-code--imenu-item-position payload)))
+    (save-excursion
+      (goto-char pos)
+      (ai-code--extract-symbol-from-line
+       (buffer-substring-no-properties
+        (line-beginning-position)
+        (line-end-position))))))
+
+(defun ai-code--imenu-noise-name-p (name)
+  "Return non-nil when NAME looks like an imenu group/template label."
+  (or (not (stringp name))
+      (string-empty-p (string-trim name))
+      (string-match-p "\\`\\*.*\\*\\'" name)
+      (string-match-p "\\`[0-9]+\\'" name)))
+
+(defun ai-code--normalize-imenu-symbol-name (name payload)
+  "Normalize imenu NAME using PAYLOAD as fallback source."
+  (let ((trimmed (and (stringp name) (string-trim name))))
+    (if (and trimmed (not (ai-code--imenu-noise-name-p trimmed)))
+        trimmed
+      (ai-code--imenu-symbol-from-position payload))))
+
 (defun ai-code--flatten-imenu-index (index)
-  "Flatten imenu INDEX alist into a list of strings."
+  "Flatten imenu INDEX into a list of useful symbol names."
   (let (result)
     (dolist (item index)
       (when (consp item)
         (let ((name (car item))
               (payload (cdr item)))
-          (cond
-           ((and (listp payload) (consp (car payload)))
-            ;; Nested list (category or sub-index)
-            (setq result (append result (ai-code--flatten-imenu-index payload))))
-           ((stringp name)
-            (push name result))))))
+          (if (ai-code--imenu-subalist-p payload)
+              (setq result (append result (ai-code--flatten-imenu-index payload)))
+            (when-let ((symbol (ai-code--normalize-imenu-symbol-name name payload)))
+              (push symbol result))))))
     result))
 
 (defun ai-code--get-functions-from-buffer (buffer)
@@ -180,6 +242,47 @@ The current buffer's file is always first."
   "Return non-nil when point is inside a comment."
   (nth 4 (syntax-ppss)))
 
+(defun ai-code--hash-completion-target-file (&optional end-pos)
+  "Return an absolute file path for @relative path ending at END-POS.
+END-POS defaults to the current '#' position."
+  (when-let ((git-root (magit-toplevel)))
+    (let* ((end (or end-pos (1- (point))))
+           (start (save-excursion
+                    (goto-char end)
+                    (skip-chars-backward "A-Za-z0-9_./-")
+                    (point)))
+           (has-at (save-excursion
+                     (goto-char start)
+                     (eq (char-before) ?@)))
+           (relative-path (and has-at
+                               (< start end)
+                               (buffer-substring-no-properties start end))))
+      (when relative-path
+        (let ((file (expand-file-name relative-path git-root))
+              (git-root-truename (file-truename git-root)))
+          (when (and (file-regular-p file)
+                     (string-prefix-p git-root-truename (file-truename file)))
+            file))))))
+
+(defun ai-code--file-symbol-candidates (file)
+  "Return sorted function/class symbol candidates from FILE."
+  (let (symbols)
+    (with-current-buffer (find-file-noselect file t)
+      (condition-case nil
+          (let ((imenu-auto-rescan t)
+                (index (imenu--make-index-alist t)))
+            (setq symbols (ai-code--flatten-imenu-index index)))
+        (error nil)))
+    (sort (delete-dups (cl-remove-if-not #'stringp symbols)) #'string<)))
+
+(defun ai-code--choose-symbol-from-file (file)
+  "Prompt user to select a symbol from FILE and return it."
+  (let ((candidates (ai-code--file-symbol-candidates file)))
+    (when candidates
+      (condition-case nil
+          (completing-read "Symbol: " candidates nil nil)
+        (quit nil)))))
+
 (defun ai-code--comment-filepath-capf ()
   "Provide completion candidates for @file paths inside comments."
   (when (and ai-code-prompt-filepath-completion-enabled
@@ -198,33 +301,47 @@ The current buffer's file is always first."
             (list start end candidates :exclusive 'no)))))))
 
 (defun ai-code--comment-auto-trigger-filepath-completion ()
-  "Auto trigger file path completion in comments when '@' is inserted."
+  "Auto trigger file path/symbol completion in comments."
   (when (and ai-code-prompt-filepath-completion-enabled
              (ai-code--comment-context-p)
              (buffer-file-name)
-             (not (minibufferp))
-             (eq (char-before) ?@))
-    (let ((candidates (ai-code--prompt-filepath-candidates)))
-      (when candidates
-        (let ((choice (completing-read "File: " candidates nil nil)))
-          (when (and choice (not (string-empty-p choice)))
-            (delete-char -1)  ; Remove the '@' we just typed
-            (insert choice)))))))
+             (not (minibufferp)))
+    (pcase (char-before)
+      (?@
+       (let ((candidates (ai-code--prompt-filepath-candidates)))
+         (when candidates
+           (let ((choice (completing-read "File: " candidates nil nil)))
+             (when (and choice (not (string-empty-p choice)))
+               (delete-char -1)  ; Remove the '@' we just typed
+               (insert choice))))))
+      (?#
+       (when-let* ((file (ai-code--hash-completion-target-file (1- (point))))
+                   (symbol (ai-code--choose-symbol-from-file file)))
+         (when (not (string-empty-p symbol))
+           (delete-char -1)  ; Remove the '#' we just typed
+           (insert (concat "#" symbol))))))))
 
 (defun ai-code--session-auto-trigger-filepath-completion ()
-  "Auto trigger file path completion in AI session buffers when '@' is inserted."
+  "Auto trigger file path/symbol completion in AI session buffers."
   (when (and ai-code-prompt-filepath-completion-enabled
              (fboundp 'ai-code-backends-infra--session-buffer-p)
              (ai-code-backends-infra--session-buffer-p (current-buffer))
              (not (minibufferp))
-             (eq (char-before) ?@)
              (magit-toplevel))
-    (let ((candidates (ai-code--prompt-filepath-candidates)))
-      (when candidates
-        (let ((choice (completing-read "File: " candidates nil nil)))
-          (when (and choice (not (string-empty-p choice)))
-            (ai-code-backends-infra--terminal-send-backspace)
-            (ai-code-backends-infra--terminal-send-string choice)))))))
+    (pcase (char-before)
+      (?@
+       (let ((candidates (ai-code--prompt-filepath-candidates)))
+         (when candidates
+           (let ((choice (completing-read "File: " candidates nil nil)))
+             (when (and choice (not (string-empty-p choice)))
+               (ai-code-backends-infra--terminal-send-backspace)
+               (ai-code-backends-infra--terminal-send-string choice))))))
+      (?#
+       (when-let* ((file (ai-code--hash-completion-target-file (1- (point))))
+                   (symbol (ai-code--choose-symbol-from-file file)))
+         (when (not (string-empty-p symbol))
+           (ai-code-backends-infra--terminal-send-backspace)
+           (ai-code-backends-infra--terminal-send-string (concat "#" symbol))))))))
 
 (defun ai-code--session-handle-at-input ()
   "Handle '@' input in AI session buffers with optional filepath completion."
@@ -246,6 +363,23 @@ The current buffer's file is always first."
               (ai-code-backends-infra--terminal-send-backspace)
               (ai-code-backends-infra--terminal-send-string choice))))))))
 
+(defun ai-code--session-handle-hash-input ()
+  "Handle '#' input in AI session buffers with optional symbol completion."
+  (interactive)
+  (let* ((should-complete
+          (and ai-code-prompt-filepath-completion-enabled
+               (fboundp 'ai-code-backends-infra--session-buffer-p)
+               (ai-code-backends-infra--session-buffer-p (current-buffer))
+               (not (minibufferp))
+               (magit-toplevel)))
+         (file (and should-complete
+                    (ai-code--hash-completion-target-file (point)))))
+    (ai-code-backends-infra--terminal-send-string "#")
+    (when-let ((symbol (and file (ai-code--choose-symbol-from-file file))))
+      (when (not (string-empty-p symbol))
+        (ai-code-backends-infra--terminal-send-backspace)
+        (ai-code-backends-infra--terminal-send-string (concat "#" symbol))))))
+
 (defun ai-code--comment-filepath-setup ()
   "Ensure comment @ completion is available in the current buffer."
   (add-hook 'completion-at-point-functions #'ai-code--comment-filepath-capf nil t))
@@ -254,6 +388,7 @@ The current buffer's file is always first."
 (define-minor-mode ai-code-prompt-filepath-completion-mode
   "Toggle @ file completion in comments and AI sessions across all buffers."
   :global t
+  :group 'ai-code
   (setq ai-code-prompt-filepath-completion-enabled
         ai-code-prompt-filepath-completion-mode)
   (if ai-code-prompt-filepath-completion-mode
@@ -287,4 +422,5 @@ The current buffer's file is always first."
            (if ai-code-prompt-filepath-completion-mode "enabled" "disabled")))
 
 (provide 'ai-code-input)
+
 ;;; ai-code-input.el ends here
