@@ -1,11 +1,11 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "ai-code" "1.30"
-  "Unified interface for AI coding CLI such as Claude Code, Codex CLI, Gemini CLI, Copilot CLI, Opencode, Grok CLI, etc."
+(define-package "ai-code" "1.45"
+  "Unified interface for AI coding CLI such as Claude Code, Codex CLI, Gemini CLI, Copilot CLI, Opencode, Grok CLI, Aider CLI, etc."
   '((emacs     "28.1")
     (transient "0.8.0")
     (magit     "2.1.0"))
   :url "https://github.com/tninja/ai-code-interface.el"
-  :commit "da92457bd6fe3c9ef408567d86ab3ca020677a55"
-  :revdesc "da92457bd6fe"
+  :commit "3bb5c3f36e899b39f69a2036d550557e0b6b4d51"
+  :revdesc "3bb5c3f36e89"
   :authors '(("Kang Tu" . "tninja@gmail.com"))
   :maintainers '(("Kang Tu" . "tninja@gmail.com")))
