@@ -15,16 +15,21 @@
 
 (defvar ai-code-use-gptel-headline)
 (defvar ai-code-prompt-suffix)
+(defvar ai-code-auto-test-type)
+(defvar ai-code-auto-test-suffix)
 (defvar ai-code-use-prompt-suffix)
 
 (declare-function yas-load-directory "yasnippet" (dir))
 (declare-function yas-minor-mode "yasnippet")
-(declare-function ai-code-cli-send-command "ai-code-interface" (command))
-(declare-function ai-code-cli-switch-to-buffer "ai-code-interface" ())
+(declare-function ai-code-cli-send-command "ai-code-backends" (command))
+(declare-function ai-code-cli-switch-to-buffer "ai-code-backends" ())
 (declare-function gptel-request "gptel" (prompt &rest args))
 (declare-function gptel-abort "gptel" (buffer))
 (declare-function ai-code--git-repo-recent-modified-files "ai-code-git" (base-dir limit))
 (declare-function ai-code--git-ignored-repo-file-p "ai-code-git" (file root))
+(declare-function ai-code--hash-completion-target-file "ai-code-input" (&optional end-pos))
+(declare-function ai-code--choose-symbol-from-file "ai-code-input" (file))
+(declare-function ai-code-current-backend-label "ai-code-backends" ())
 
 (defcustom ai-code-prompt-preprocess-filepaths t
   "When non-nil, preprocess the prompt to replace file paths.
@@ -161,8 +166,13 @@ Returns the full prompt text with suffix for sending to AI."
 
 (defun ai-code--write-prompt-to-file-and-send (prompt-text)
   "Write PROMPT-TEXT to the AI prompt file."
-  (let* ((full-prompt (concat (if (and ai-code-use-prompt-suffix ai-code-prompt-suffix)
-                                  (concat prompt-text "\n" ai-code-prompt-suffix)
+  (let* ((suffix-parts (delq nil (list ai-code-prompt-suffix
+                                       (when ai-code-auto-test-type
+                                         ai-code-auto-test-suffix))))
+         (suffix (when (and ai-code-use-prompt-suffix suffix-parts)
+                   (mapconcat #'identity suffix-parts "\n")))
+         (full-prompt (concat (if suffix
+                                  (concat prompt-text "\n" suffix)
                                 prompt-text) "\n"))
          (prompt-file (ai-code--get-ai-code-prompt-file-path))
          (original-default-directory default-directory))
@@ -216,19 +226,77 @@ NOTE: This does not handle file paths containing spaces."
   "Return FILE relative to GIT-ROOT-TRUENAME, prefixed with '@'."
   (concat "@" (file-relative-name (file-truename file) git-root-truename)))
 
-(defun ai-code--visible-window-files (git-root-truename)
-  "Return visible window file list under GIT-ROOT-TRUENAME."
+(defun ai-code--normalize-path (file)
+  "Return normalized absolute path for FILE.
+If FILE exists, return its truename. Otherwise return expanded path."
+  (let ((full (expand-file-name file)))
+    (if (file-exists-p full)
+        (file-truename full)
+      full)))
+
+(defun ai-code--candidate-path (file git-root-truename)
+  "Return completion candidate for FILE.
+Return '@'-prefixed path relative to GIT-ROOT-TRUENAME when FILE is under
+that root, otherwise return the absolute path."
+  (let ((full-truename (ai-code--normalize-path file)))
+    (if (string-prefix-p git-root-truename full-truename)
+        (ai-code--relative-filepath full-truename git-root-truename)
+      full-truename)))
+
+(defun ai-code--current-frame-dired-paths (git-root-truename)
+  "Return dired directory candidates from current frame under GIT-ROOT-TRUENAME."
+  (let ((paths '()))
+    (dolist (win (window-list nil 'no-minibuffer))
+      (with-current-buffer (window-buffer win)
+        (when (derived-mode-p 'dired-mode)
+          (let ((dir (if (fboundp 'dired-current-directory)
+                         (dired-current-directory)
+                       default-directory)))
+            (when (and dir
+                       (file-directory-p dir)
+                       (string-prefix-p git-root-truename
+                                        (file-truename dir))
+                       (not (ai-code--git-ignored-repo-file-p
+                             dir
+                             git-root-truename)))
+              (push (ai-code--relative-filepath dir
+                                                git-root-truename)
+                    paths))))))
+    (nreverse (delete-dups paths))))
+
+(defun ai-code--visible-window-files ()
+  "Return files from visible windows in current frame."
   (let ((files '())
         (selected (selected-window)))
     (dolist (win (cons selected
                        (delq selected (window-list nil 'no-minibuffer))))
-      (let* ((buf (window-buffer win))
-             (file (buffer-file-name buf)))
-        (when (and (ai-code--file-in-git-repo-p file git-root-truename)
-                   (not (ai-code--git-ignored-repo-file-p file git-root-truename)))
+      (let ((file (buffer-file-name (window-buffer win))))
+        (when file
           (push file files))))
     (nreverse (delete-dups files))))
 
+(defun ai-code--recent-buffer-paths (git-root-truename)
+  "Return candidate paths for most recent 5 visited buffer files or directories."
+  (let ((files '())
+        (count 0))
+    (dolist (buf (buffer-list))
+      (when (< count 5)
+        (with-current-buffer buf
+          (if (derived-mode-p 'dired-mode)
+              (let ((dir (if (fboundp 'dired-current-directory)
+                             (dired-current-directory)
+                           default-directory)))
+                (when dir
+                  (push dir files)
+                  (setq count (1+ count))))
+            (let ((file (buffer-file-name buf)))
+              (when file
+                (push file files)
+                (setq count (1+ count))))))))
+    (mapcar (lambda (file)
+              (ai-code--candidate-path file git-root-truename))
+            (nreverse files))))
+
 (defun ai-code--buffer-file-list (git-root-truename &optional skip-files)
   "Return buffer file list under GIT-ROOT-TRUENAME, skipping SKIP-FILES."
   (let ((files '()))
@@ -259,21 +327,29 @@ NOTE: This does not handle file paths containing spaces."
   (when-let ((git-root (magit-toplevel)))
     (let* ((git-root-truename (file-truename git-root))
            (current-file (buffer-file-name (current-buffer)))
-           (visible-files (ai-code--visible-window-files git-root-truename))
-           (skip-files (mapcar #'file-truename visible-files))
+           (current-frame-dired-paths
+            (ai-code--current-frame-dired-paths git-root-truename))
+           (visible-files (ai-code--visible-window-files))
+           (skip-files (mapcar #'ai-code--normalize-path visible-files))
            (buffer-files (ai-code--buffer-file-list git-root-truename skip-files))
            (recent-files (ai-code--repo-recent-files git-root-truename))
            (ignore-prefix (concat "@" ai-code-files-dir-name "/"))
            (visible-paths (mapcar (lambda (file)
-                                    (ai-code--relative-filepath file git-root-truename))
+                                    (ai-code--candidate-path file git-root-truename))
                                   visible-files))
+           (recent-buffer-paths
+            (ai-code--recent-buffer-paths git-root-truename))
            (buffer-paths (mapcar (lambda (file)
-                                   (ai-code--relative-filepath file git-root-truename))
+                                   (ai-code--candidate-path file git-root-truename))
                                  buffer-files))
            (recent-paths (mapcar (lambda (file)
-                                   (ai-code--relative-filepath file git-root-truename))
+                                   (ai-code--candidate-path file git-root-truename))
                                  recent-files))
-           (combined (append visible-paths buffer-paths recent-paths))
+           (combined (append current-frame-dired-paths
+                             visible-paths
+                             recent-buffer-paths
+                             buffer-paths
+                             recent-paths))
            (deduped (ai-code--dedupe-preserve-order combined))
            (filtered '()))
       (dolist (item deduped)
@@ -297,15 +373,25 @@ NOTE: This does not handle file paths containing spaces."
             (list start end candidates :exclusive 'no)))))))
 
 (defun ai-code--prompt-auto-trigger-filepath-completion ()
-  "Auto trigger file path completion when '@' is inserted."
-  (when (and (not (minibufferp))
-             (eq (char-before) ?@))
-    (let ((candidates (ai-code--prompt-filepath-candidates)))
-      (when candidates
-        (let ((choice (completing-read "File: " candidates nil nil)))
-          (when (and choice (not (string-empty-p choice)))
-            (delete-char -1)  ; Remove the '@' we just typed
-            (insert choice)))))))
+  "Auto trigger file path/symbol completion when '@' or '#' is inserted."
+  (when (not (minibufferp))
+    (pcase (char-before)
+      (?@
+       (let ((candidates (ai-code--prompt-filepath-candidates)))
+         (when candidates
+           (let ((choice (completing-read "File: " candidates nil nil)))
+             (when (and choice (not (string-empty-p choice)))
+               (delete-char -1)  ; Remove the '@' we just typed
+               (insert choice))))))
+      (?#
+       (require 'ai-code-input nil t)
+       (when (and (fboundp 'ai-code--hash-completion-target-file)
+                  (fboundp 'ai-code--choose-symbol-from-file))
+         (when-let* ((file (ai-code--hash-completion-target-file (1- (point))))
+                     (symbol (ai-code--choose-symbol-from-file file)))
+           (when (not (string-empty-p symbol))
+             (delete-char -1)  ; Remove the '#' we just typed
+             (insert (concat "#" symbol)))))))))
 
 (defun ai-code--insert-prompt (prompt-text)
   "Preprocess and insert PROMPT-TEXT into the AI prompt file.
@@ -384,8 +470,8 @@ Otherwise, return the current `default-directory`."
   "Generate a task filename from TASK-NAME.
 If `ai-code-task-use-gptel-filename` is non-nil, use GPTel to generate
 a smart filename. Otherwise, use cleaned-up task name directly.
-If TASK-NAME contains 'rdar://ID', use 'rdar_ID_' as prefix.
-Otherwise, use 'task_YYYYMMDD_' as prefix.
+If TASK-NAME contains `rdar://ID`, use `rdar_ID_` as prefix.
+Otherwise, use `task_YYYYMMDD_` as prefix.
 Returns a filename with .org suffix."
   (let* ((radar-id (when (string-match "rdar://\\([0-9]+\\)" task-name)
                      (match-string 1 task-name)))
@@ -410,6 +496,50 @@ Returns a filename with .org suffix."
       (setq generated-name (substring generated-name 0 60)))
     (concat prefix generated-name ".org")))
 
+(defun ai-code--initialize-task-file-content (task-name task-url)
+  "Insert initial task content using TASK-NAME and TASK-URL."
+  (insert (format "#+TITLE: %s\n" task-name))
+  (insert (format "#+DATE: %s\n" (format-time-string "%Y-%m-%d")))
+  (unless (string-empty-p task-url)
+    (insert (format "#+URL: %s\n" task-url)))
+  (let ((label (ai-code-current-backend-label)))
+    (insert (format "#+AGENT: %s\n" label))
+    (insert "#+SESSION_ID: <Usually you can get the session id with /status or /stat in AI coding window>\n"))
+  (insert "\n* Task Description\n\n")
+  (insert task-name)
+  (insert "\n\n* Investigation\n\n")
+  (insert "# Enter your prompts here. After that,\n# Select them and use C-c a SPC (ai-code-send-command) to send to AI\n\n")
+  (insert "# Use C-c a n (ai-code-take-notes) to copy notes back from AI session\n\n")
+  (insert "\n\n* Code Change\n\n"))
+
+(defun ai-code--open-or-create-task-file (task-file confirmed-filename task-name task-url)
+  "Open TASK-FILE and initialize it when needed.
+CONFIRMED-FILENAME determines if .org should be appended.
+TASK-NAME and TASK-URL are used to initialize new files."
+  (unless (string-suffix-p ".org" confirmed-filename)
+    (setq task-file (concat task-file ".org")))
+  (find-file-other-window task-file)
+  (unless (file-exists-p task-file)
+    (ai-code--initialize-task-file-content task-name task-url))
+  (message "Opened task file: %s" task-file))
+
+(defun ai-code--select-task-target-directory (ai-code-files-dir current-dir)
+  "Prompt user to select target directory.
+
+AI-CODE-FILES-DIR is the path to the .ai.code.files directory.
+CURRENT-DIR is the current default directory.
+
+Returns the selected directory path."
+  (let ((target-dir (completing-read
+                     "Create task file in: "
+                     (list (format "ai-code-files-dir: %s" ai-code-files-dir)
+                           (format "current directory: %s" current-dir))
+                     nil t nil nil
+                     (format "ai-code-files-dir: %s" ai-code-files-dir))))
+    (if (string-prefix-p "ai-code-files-dir:" target-dir)
+        ai-code-files-dir
+      current-dir)))
+
 ;;;###autoload
 (defun ai-code-create-or-open-task-file ()
   "Create or open an AI task file.
@@ -419,33 +549,24 @@ using GPTel, and creates the task file."
   (interactive)
   (let ((task-name (read-string "Task name (empty to open task directory): ")))
     (if (string-empty-p task-name)
-        ;; Open the task directory
         (let ((ai-code-files-dir (ai-code--ensure-files-directory)))
           (dired-other-window ai-code-files-dir)
           (message "Opened task directory: %s" ai-code-files-dir))
-      ;; Create a new task file
       (let* ((task-url (read-string "URL (optional, press Enter to skip): "))
              (ai-code-files-dir (ai-code--ensure-files-directory))
              (generated-filename (ai-code--generate-task-filename task-name))
-             (confirmed-filename (read-string "Confirm task filename: " generated-filename))
-             (task-file (expand-file-name confirmed-filename ai-code-files-dir)))
-        ;; Ensure filename has .org extension
-        (unless (string-suffix-p ".org" confirmed-filename)
-          (setq task-file (concat task-file ".org")))
-        (find-file-other-window task-file)
-        (unless (file-exists-p task-file)
-          ;; Initialize new task file
-          (insert (format "#+TITLE: %s\n" task-name))
-          (insert (format "#+DATE: %s\n" (format-time-string "%Y-%m-%d")))
-          (unless (string-empty-p task-url)
-            (insert (format "#+URL: %s\n" task-url)))
-          (insert "\n* Task Description\n\n")
-          (insert task-name)
-          (insert "\n\n* Investigation\n\n")
-          (insert "# Enter your prompts here. After that,\n# Select them and use C-c a SPC (ai-code-send-command) to send to AI\n\n")
-          (insert "# Use C-c a n (ai-code-take-notes) to copy notes back from AI session\n\n")
-          (insert "\n\n* Code Change\n\n"))
-        (message "Opened task file: %s" task-file)))))
+             (confirmed-filename (read-string "Confirm task filename (end with / to create subdirectory): " generated-filename))
+             (current-dir (expand-file-name default-directory))
+             (selected-dir (ai-code--select-task-target-directory ai-code-files-dir current-dir))
+             (create-dir-only-p (string-suffix-p "/" confirmed-filename))
+             (task-file (expand-file-name confirmed-filename selected-dir)))
+        (if create-dir-only-p
+            (let ((subdir (expand-file-name (directory-file-name confirmed-filename) selected-dir)))
+              (unless (file-directory-p subdir)
+                (make-directory subdir t))
+              (dired-other-window subdir)
+              (message "Opened directory: %s" subdir))
+          (ai-code--open-or-create-task-file task-file confirmed-filename task-name task-url))))))
 
 ;;;###autoload
 (add-to-list 'auto-mode-alist
