@@ -1,8 +1,8 @@
-;;; ai-code.el --- Unified interface for AI coding CLI such as Claude Code, Codex CLI, Gemini CLI, Copilot CLI, Opencode, Grok CLI, etc -*- lexical-binding: t; -*-
+;;; ai-code.el --- Unified interface for AI coding CLI such as Claude Code, Codex CLI, Gemini CLI, Copilot CLI, Opencode, Grok CLI, Aider CLI, etc -*- lexical-binding: t; -*-
 
 ;; Author: Kang Tu <tninja@gmail.com>
-;; Package-Version: 1.30
-;; Package-Revision: da92457bd6fe
+;; Package-Version: 1.45
+;; Package-Revision: 3bb5c3f36e89
 ;; Package-Requires: ((emacs "28.1") (transient "0.8.0") (magit "2.1.0"))
 ;; URL: https://github.com/tninja/ai-code-interface.el
 
@@ -25,16 +25,22 @@
 ;;   - Grok CLI
 ;;   - Cursor CLI
 ;;   - Kiro CLI
+;;   - CodeBuddy Code CLI
+;;   - Aider CLI
 ;;
 ;; Basic configuration example:
 ;;
 ;; (use-package ai-code
 ;;   :config
-;;   ;; use codex as backend, other options are 'gemini, 'github-copilot-cli, 'opencode, 'grok, 'claude-code-ide, 'claude-code-el, 'claude-code, 'cursor, 'kiro
+;;   ;; use codex as backend, other options are 'gemini, 'github-copilot-cli, 'opencode, 'grok, 'claude-code-ide, 'claude-code-el, 'claude-code, 'cursor, 'kiro, 'codebuddy, 'aider
 ;;   (ai-code-set-backend 'codex) ;; set your preferred backend
 ;;   (global-set-key (kbd "C-c a") #'ai-code-menu)
 ;;   ;; Optional: Enable @ file completion in comments and AI sessions
 ;;   (ai-code-prompt-filepath-completion-mode 1)
+;;   ;; Optional: Ask AI to run test after code changes, for a tighter build-test loop
+;;   (setq ai-code-auto-test-type 'test-after-change)
+;;   ;; Optional: In the AI session buffer (Evil normal state), SPC triggers the prompt entry UI
+;;   (with-eval-after-load 'evil (ai-code-backends-infra-evil-setup))
 ;;   (global-auto-revert-mode 1)
 ;;   (setq auto-revert-interval 1) ;; set to 1 second for faster update
 ;;   )
@@ -43,7 +49,7 @@
 ;;   - Transient-driven Hub (C-c a) for all AI capabilities.
 ;;   - One key switching to different AI backend (C-c a s).
 ;;   - Context-aware code actions (change code, implement TODOs, explain code, @ completion).
-;;   - Agile development workflows (TDD cycle, refactoring navigator, review helper).
+;;   - Agile development workflows (TDD cycle, refactoring navigator, review helper, Build / Test feedback loop).
 ;;   - Seamless prompt management using Org-mode.
 ;;   - AI-assisted bash commands and productivity utilities.
 ;;   - Multiple AI coding sessions management.
@@ -60,6 +66,10 @@
 (require 'transient)
 (require 'seq)
 
+(defgroup ai-code nil
+  "Unified interface for multiple AI coding CLIs."
+  :group 'tools)
+
 (require 'ai-code-backends)
 (require 'ai-code-backends-infra)
 (require 'ai-code-input)
@@ -69,9 +79,11 @@
 (require 'ai-code-change)
 (require 'ai-code-discussion)
 (require 'ai-code-codex-cli)
+(require 'ai-code-aider-cli)
 (require 'ai-code-github-copilot-cli)
 (require 'ai-code-opencode)
 (require 'ai-code-grok-cli)
+(require 'ai-code-codebuddy-cli)
 (require 'ai-code-file)
 (require 'ai-code-ai)
 (require 'ai-code-notifications)
@@ -85,6 +97,7 @@
 (declare-function ai-code-set-backend "ai-code-backends")
 (declare-function ai-code-select-backend "ai-code-backends")
 (declare-function ai-code-open-backend-config "ai-code-backends")
+(declare-function ai-code-open-backend-agent-file "ai-code-backends")
 (declare-function ai-code-upgrade-backend "ai-code-backends")
 (declare-function ai-code-backends-infra--session-buffer-p "ai-code-backends-infra" (buffer))
 
@@ -108,11 +121,60 @@ with a newline separator."
   :type '(choice (const nil) string)
   :group 'ai-code)
 
+;;;###autoload
 (defcustom ai-code-use-prompt-suffix t
   "When non-nil, append `ai-code-prompt-suffix` where supported."
   :type 'boolean
   :group 'ai-code)
 
+;;;###autoload
+(defcustom ai-code-test-after-code-change-suffix
+  "If any program code changes, run unit-tests and follow up on the test-result (fix code if there is an error)."
+  "User-provided prompt suffix for test-after-code-change."
+  :type '(choice (const nil) string)
+  :group 'ai-code)
+
+;;;###autoload
+(defvar ai-code-auto-test-suffix ai-code-test-after-code-change-suffix
+  "Default prompt suffix to request running tests after code changes.")
+
+(defun ai-code--test-after-code-change--resolve-tdd-suffix ()
+  "Return the TDD-style suffix for test-after-code-change prompts."
+  (let ((pattern (when (and (boundp 'ai-code--tdd-test-pattern-instruction)
+                            (stringp ai-code--tdd-test-pattern-instruction))
+                   ai-code--tdd-test-pattern-instruction)))
+    (concat "Follow TDD principles - write the failing test first, then implement the minimal code to make it pass. Only update test and source code. Run the tests and follow up with the test result (fix code if there is error)."
+            (or pattern ""))))
+
+(defun ai-code--test-after-code-change--set (symbol value)
+  "Set SYMBOL to VALUE and update related suffix behavior."
+  (set-default symbol value)
+  (set symbol value)
+  (pcase value
+    ('test-after-change
+     (setq ai-code-auto-test-suffix
+           ai-code-test-after-code-change-suffix))
+    ('tdd
+     (setq ai-code-auto-test-suffix
+           (ai-code--test-after-code-change--resolve-tdd-suffix)))
+    (_
+     (setq ai-code-auto-test-suffix nil))))
+
+(defun ai-code--apply-auto-test-type (value)
+  "Set `ai-code-auto-test-type` and refresh related suffix."
+  (setq ai-code-auto-test-type value)
+  (ai-code--test-after-code-change--set 'ai-code-auto-test-type value)
+  value)
+
+(defcustom ai-code-auto-test-type nil
+  "Select how prompts request tests after code changes."
+  :type '(choice (const :tag "Use test after code change prompt" test-after-change)
+                 (const :tag "Use TDD Red+Green prompt" tdd)
+                 (const :tag "Off" nil))
+  :set #'ai-code--test-after-code-change--set
+  :group 'ai-code)
+
+
 ;;;###autoload
 (defcustom ai-code-cli "claude"
   "The command-line AI tool to use for `ai-code-apply-prompt-on-current-file`."
@@ -200,6 +262,31 @@ Otherwise switch to AI CLI buffer."
   :reader (lambda (_prompt _initial-input _history)
             (not ai-code-use-prompt-suffix)))
 
+(defclass ai-code--code-change-auto-test-type (transient-lisp-variable)
+  ((variable :initform 'ai-code-auto-test-type)
+   (format :initform "%k %d %v")
+   (reader :initform #'transient-lisp-variable--read-value))
+  "Selection helper for `ai-code-auto-test-type`.")
+
+(transient-define-infix ai-code--infix-select-code-change-auto-test ()
+  "Select `ai-code-auto-test-type` mode."
+  :class 'ai-code--code-change-auto-test-type
+  :key "T"
+  :description "Auto test type:"
+  :reader (lambda (_prompt _initial-input _history)
+            (let* ((choices '(("Use test after code change prompt" . test-after-change)
+                              ("Use TDD Red+Green prompt" . tdd)
+                              ("Off" . nil)))
+                   (choice (completing-read "Test after code change: "
+                                            (mapcar #'car choices)
+                                            nil t)))
+              (let ((value (cdr (assoc choice choices))))
+                (ai-code--apply-auto-test-type value)
+                (message "Auto test type set to %s; prompt suffix is now %s"
+                         (or value "off")
+                         (or ai-code-auto-test-suffix "cleared"))
+                value))))
+
 (defun ai-code--select-backend-description (&rest _)
   "Dynamic description for the Select Backend menu item.
 Shows the current backend label to the right."
@@ -211,13 +298,15 @@ Shows the current backend label to the right."
   ["AI Code Commands"
    ["AI CLI session"
     ("a" "Start AI CLI (C-u: args)" ai-code-cli-start)
-    ("R" "Resume AI CLI" ai-code-cli-resume)
+    ("R" "Resume AI CLI (C-u: args)" ai-code-cli-resume)
     ("z" "Switch to AI CLI (C-u: hide)" ai-code-cli-switch-to-buffer-or-hide)
     ;; Use plist style to provide a dynamic description function.
     ("s" ai-code-select-backend :description ai-code--select-backend-description)
     ("u" "Install / Upgrade AI CLI" ai-code-upgrade-backend)
     ("g" "Open backend config (eg. add mcp)" ai-code-open-backend-config)
-    ("|" "Apply prompt on file" ai-code-apply-prompt-on-current-file)]
+    ("G" "Open backend repo agent file" ai-code-open-backend-agent-file)
+    ("|" "Apply prompt on file" ai-code-apply-prompt-on-current-file)
+    ]
 
    ["AI Code Actions"
     (ai-code--infix-toggle-suffix)
@@ -227,64 +316,42 @@ Shows the current backend label to the right."
     ("x" "Explain code in scope" ai-code-explain)
     ("<SPC>" "Send command (C-u: context)" ai-code-send-command)
     ("@" "Add context (C-u: clear)" ai-code-context-action)
-    ("K" "Create or open task file" ai-code-create-or-open-task-file)]
+    ]
 
    ["AI Agile Development"
+    (ai-code--infix-select-code-change-auto-test)
     ("r" "Refactor Code"               ai-code-refactor-book-method)
     ("t" "Test Driven Development"     ai-code-tdd-cycle)
     ("v" "Pull or Review Code Change"  ai-code-pull-or-review-diff-file)
     ;; ("b" "Send prompt block to AI" ai-code-prompt-send-block)
     ("!" "Run Current File or Command" ai-code-run-current-file-or-shell-cmd)
-    ("b" "Build project"               ai-code-build-project)
-    ("p" "Open prompt history file" ai-code-open-prompt-file)
-    ("I" "Insert function name at point" ai-code-insert-function-at-point)]
+    ("b" "Build / Test project"               ai-code-build-or-test-project)
+    ;; ("I" "Insert function name at point" ai-code-insert-function-at-point)
+    ("K" "Create or open task file" ai-code-create-or-open-task-file)
+    ("n" "Take notes from AI session region" ai-code-take-notes)
+    ]
 
    ["Other Tools"
     ("." "Init projectile and gtags" ai-code-init-project)
     ("e" "Debug exception (C-u: clipboard)" ai-code-investigate-exception)
     ("f" "Fix Flycheck errors in scope" ai-code-flycheck-fix-errors-in-scope)
     ("k" "Copy Cur File Name (C-u: full)" ai-code-copy-buffer-file-name-to-clipboard)
-    ("d" "Toggle current buffer dedicated" ai-code-toggle-current-buffer-dedicated)
+    ;; ("d" "Toggle current buffer dedicated" ai-code-toggle-current-buffer-dedicated)
     ("o" "Open recent file (C-u: insert)" ai-code-git-repo-recent-modified-files)
     ;; ("o" "Open Clipboard file dir" ai-code-open-clipboard-file-path-as-dired)
+    ("p" "Open prompt history file" ai-code-open-prompt-file)
     ("m" "Debug python MCP server" ai-code-debug-mcp)
-    ("n" "Take notes from AI session region" ai-code-take-notes)
     ("N" "Toggle notifications" ai-code-notifications-toggle)]])
 
 
 
-;; When in a special buffer (e.g., *claude-code*) and using evil-mode,
-;; pressing SPC in normal state will send the prompt.
+;; DONE: When in an ai-coding session (e.g., with -backends-infra.el code) and using evil-mode,
+;; pressing SPC in normal state will trigger ai-code-send-command.
+;; Fixed implementation moved to ai-code-backends-infra.el:
+;;   - `ai-code-backends-infra--evil-spc-command' - intercepts SPC in AI session buffers
+;;   - `ai-code-backends-infra-evil-setup' - call with (with-eval-after-load 'evil ...)
 
-;; following code is buggy
-(defvar ai-code--original-spc-command-in-evil-normal-state nil
-  "Original command for SPC in `evil-normal-state`.")
-
-(defun ai-code-spc-command-for-special-buffer-in-evil ()
-  "In special buffers (*...*), run `ai-code-send-command`.
-Otherwise, run the original command for SPC."
-  (interactive)
-  (if (and (string-prefix-p "*" (buffer-name))
-           (string-suffix-p "*" (buffer-name)))
-      (call-interactively #'ai-code-send-command)
-    (when ai-code--original-spc-command-in-evil-normal-state
-      (call-interactively ai-code--original-spc-command-in-evil-normal-state))))
-
-;;;###autoload
-(defun ai-code-evil-setup ()
-  "Setup AI Code integration with Evil mode.
-This function configures SPC key binding in Evil normal state for
-special AI Code buffers.  Call this function after Evil is loaded,
-typically in your Emacs configuration with:
-  (with-eval-after-load \\='evil (ai-code-evil-setup))"
-  (interactive)
-  (when (and (featurep 'evil) (boundp 'evil-normal-state-map))
-    (unless ai-code--original-spc-command-in-evil-normal-state
-      (setq ai-code--original-spc-command-in-evil-normal-state
-            (lookup-key evil-normal-state-map (kbd "SPC"))))
-    (when ai-code--original-spc-command-in-evil-normal-state
-      (define-key evil-normal-state-map (kbd "SPC")
-        #'ai-code-spc-command-for-special-buffer-in-evil))))
+;; (with-eval-after-load 'evil (ai-code-backends-infra-evil-setup))
 
 (provide 'ai-code)
 
